from django.forms import (
    BooleanField,
    Form,
    ModelChoiceField,
    ModelForm,
    TextInput,
)
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from . import GroupFormMixin
from ..models import (
    Place,
)


class EventAddPlaceForm(Form):
    required_css_class = "required"

    place = ModelChoiceField(
        label=_("Place"),
        queryset=Place.objects.all().order_by("name"),
        empty_label=_("Create a missing place"),
        required=False,
    )
    add_alias = BooleanField(initial=True, required=False)

    def __init__(self, *args, **kwargs):
        self.instance = kwargs.pop("instance", False)
        super().__init__(*args, **kwargs)
        if self.instance.location:
            self.fields["add_alias"].label = _(
                'Add "{}" to the aliases of the place'
            ).format(self.instance.location)
        else:
            self.fields.pop("add_alias")
        if self.instance.exact_location:
            self.fields["place"].initial = self.instance.exact_location
            self.fields["add_alias"].initial = False

    def modified_event(self):
        return self.cleaned_data.get("place")

    def save(self):
        if self.cleaned_data.get("place"):
            place = self.cleaned_data.get("place")
            self.instance.exact_location = place
            self.instance.save(update_fields=["exact_location"])
            if self.cleaned_data.get("add_alias"):
                if place.aliases:
                    place.aliases.append(self.instance.location.strip())
                else:
                    place.aliases = [self.instance.location.strip()]
                place.save()

        return self.instance


class PlaceForm(GroupFormMixin, ModelForm):
    required_css_class = "required"

    apply_to_all = BooleanField(
        initial=True,
        label=_(
            "On saving, use aliases to detect all matching events with missing place"
        ),
        required=False,
    )

    class Meta:
        model = Place
        fields = "__all__"
        widgets = {"location": TextInput()}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.add_group("header", _("Header"))
        self.fields["name"].group_id = "header"

        self.add_group("address", _("Address"))
        self.fields["address"].group_id = "address"
        self.fields["postcode"].group_id = "address"
        self.fields["city"].group_id = "address"
        self.fields["location"].group_id = "address"

        self.add_group("meta", _("Meta"))
        self.fields["aliases"].group_id = "meta"

        self.add_group("information", _("Information"))
        self.fields["description"].group_id = "information"

    def as_grid(self):
        result = (
            '<div class="grid"><div>'
            + super().as_p()
            + """</div><div><div class="map-widget">
            <div id="map_location"></div>
            <p>Cliquez pour ajuster la position GPS</p></div>
            <input type="checkbox" role="switch" id="lock_position">Verrouiller la position</lock>
            <script>
                document.getElementById("lock_position").onclick = function() {
                    const field = document.getElementById("id_location");
                    if (this.checked)
                        field.setAttribute("readonly", true);
                    else
                        field.removeAttribute("readonly");
                }
            </script>
            </div></div>"""
        )

        return mark_safe(result)

    def apply(self):
        return self.cleaned_data.get("apply_to_all")
