from django.forms import (
    BooleanField,
    CharField,
    EmailField,
    Form,
    ModelChoiceField,
    MultipleChoiceField,
    Textarea,
    URLField,
    ValidationError,
    formset_factory,
)
from django.utils.translation import gettext_lazy as _
from django_better_admin_arrayfield.forms.widgets import DynamicArrayWidget

from ..models import (
    Category,
    Tag,
)


class GroupFormMixin:
    template_name = "agenda_culturel/forms/div_group.html"

    class FieldGroup:
        def __init__(
            self,
            id,
            label,
            display_label=False,
            maskable=False,
            default_masked=True,
        ):
            self.id = id
            self.label = label
            self.display_label = display_label
            self.maskable = maskable
            self.default_masked = default_masked

        def toggle_field_name(self):
            return "group_" + self.id

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.groups = []

    def add_group(self, *args, **kwargs):
        self.groups.append(GroupFormMixin.FieldGroup(*args, **kwargs))
        if self.groups[-1].maskable:
            self.fields[self.groups[-1].toggle_field_name()] = BooleanField(
                required=False
            )
            self.fields[self.groups[-1].toggle_field_name()].toggle_group = True

    def get_fields_in_group(self, g):
        return [
            f
            for f in self.visible_fields()
            if not hasattr(f.field, "toggle_group")
            and hasattr(f.field, "group_id")
            and f.field.group_id == g.id
        ]

    def get_no_group_fields(self):
        return [
            f
            for f in self.visible_fields()
            if not hasattr(f.field, "toggle_group")
            and (not hasattr(f.field, "group_id") or f.field.group_id is None)
        ]

    def fields_by_group(self):
        return [(g, self.get_fields_in_group(g)) for g in self.groups] + [
            (
                GroupFormMixin.FieldGroup("other", _("Other")),
                self.get_no_group_fields(),
            )
        ]

    def clean(self):
        result = super().clean()

        if result:
            data = dict(self.data)
            # for each masked group, we remove data
            for g in self.groups:
                if g.maskable and g.toggle_field_name() not in data:
                    fields = self.get_fields_in_group(g)
                    for f in fields:
                        self.cleaned_data[f.name] = None

        return result


class SimpleContactForm(GroupFormMixin, Form):
    email = EmailField(
        label=_("Your email"),
        help_text=_("Your email address"),
        max_length=254,
        required=False,
    )

    comments = CharField(
        label=_("Comments"),
        help_text=_(
            "Your message for the moderation team (comments, clarifications, requests...)"
        ),
        widget=Textarea,
        max_length=2048,
        required=False,
    )

    def __init__(self, *args, **kwargs):
        is_authenticated = "is_authenticated" in kwargs and kwargs["is_authenticated"]
        super().__init__(*args, **kwargs)

        if not is_authenticated:
            self.add_group(
                "communication",
                _(
                    "Receive notification of publication or leave a message for moderation"
                ),
                maskable=True,
                default_masked=True,
            )
            self.fields["email"].group_id = "communication"
            self.fields["comments"].group_id = "communication"
        else:
            del self.fields["email"]
            del self.fields["comments"]


class URLSubmissionSimpleForm(Form):
    url = URLField(max_length=512)


class URLSubmissionForm(GroupFormMixin, Form):
    required_css_class = "required"

    url = URLField(max_length=512)
    category = ModelChoiceField(
        label=_("Category"),
        queryset=Category.objects.all().order_by("name"),
        initial=None,
        required=False,
    )
    tags = MultipleChoiceField(
        label=_("Tags"), initial=None, choices=[], required=False
    )

    def __init__(self, *args, **kwargs):
        kwargs.pop("is_authenticated", False)
        super().__init__(*args, **kwargs)
        self.fields["tags"].choices = Tag.get_tag_groups(all=True)

        self.add_group("event", _("Event"))
        self.fields["url"].group_id = "event"
        self.fields["category"].group_id = "event"
        self.fields["tags"].group_id = "event"


class URLSubmissionFormWithContact(SimpleContactForm, URLSubmissionForm):
    pass


URLSubmissionFormSet = formset_factory(URLSubmissionForm, extra=9, min_num=1)


class DynamicArrayWidgetURLs(DynamicArrayWidget):
    template_name = "agenda_culturel/widgets/widget-urls.html"


class DynamicArrayWidgetTags(DynamicArrayWidget):
    template_name = "agenda_culturel/widgets/widget-tags.html"


class MultipleChoiceFieldAcceptAll(MultipleChoiceField):
    def validate(self, value):
        # check if each element is without "/"
        for item in value:
            if "/" in item:
                raise ValidationError(_("The '/' character is not allowed."))
